#include <net-snmp/net-snmp-config.h>

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif
#include <sys/types.h>
#if HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif
#if TIME_WITH_SYS_TIME
# ifdef WIN32
#  include <sys/timeb.h>
# else
#  include <sys/time.h>
# endif
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#if HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#include <stdio.h>
#if HAVE_WINSOCK_H
#include <winsock.h>
#endif
#if HAVE_NETDB_H
#include <netdb.h>
#endif
#if HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif


#include <net-snmp/net-snmp-includes.h>
#include <mysql.h>

#ifndef USERTRACKERH
#define USERTRACKERH

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <syslog.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef LINUX
	#include <getopt.h>
#endif

#define MAXSTR 255

//#define debug

#define LISTE "/var/log/tracker_ip_list" 		// IP listesinin yazilacagi liste
#define CEVAPVAR "/var/log/tracker_alive_ips" 	// Cevap gelen IP'lerin yazilacagi dosya
#define CEVAPYOK "/var/log/tracker_dead_ips" 	// Cevap gelmeyen IP'lerin yazilacagi dosya

struct ayr {
	char MTAHost[51];
	char FromName[51];
	char To1[51];
	char To2[51];
	char To3[51];
	char Subject[51];
	int Period;
	int TrackerPeriod; // When the tracking will start after the last run ?
	char Status[9]; // running or sleeping
	char Tracking[4];
	char LastUpdateTime[17];
	char LastUpdateFinishTime[17];
	char SnmpRO[11];
	char SnmpRW[11];
	char sqlhost[127];
	char sqluser[20];
	char sqlpass[20];
	char sqldb[20];
	char ldaphost[250];
	char ldapbase[250];
	char ldapmanager[25];
	char ldapmanagerpass[250];
};

#define NETSNMP_DS_WALK_INCLUDE_REQUESTED	1
#define NETSNMP_DS_WALK_PRINT_STATISTICS	2
#define NETSNMP_DS_WALK_DONT_CHECK_LEXICOGRAPHIC	3

#define MAX_VLAN_NUMBER 200 /* Total active VLAN number in a switch */
#define MAX_DEGER_SAYISI 2048
#define LOGFACILITY LOG_LOCAL1
#define CONFIG_FILE "/etc/trackerd.conf"
#define SQLPORT 3306

// Switch'deki operational VLAN listesini verir
#define VLAN_LIST ".1.3.6.1.4.1.9.9.46.1.3.1.1.2"

// community_ismi@VLAN_NBR ile bu VLAN'deki MAC listesini verir
// Her VLAN icin bu listeyi ayri ayri almak gerekiyor
#define MAC_LIST ".1.3.6.1.2.1.17.4.3.1.1"

// Her VLAN icin bridge port numaralarini verir
// community_ismi@VLAN_NBR seklinde tum VLAN'ler icin liste almak gerekiyor
#define BRIDGE_PORT_NUMBERS ".1.3.6.1.2.1.17.4.3.1.2"


// index mapping
#define IFINDEX ".1.3.6.1.2.1.17.1.4.1.2"


// SNMP ifindex isim listesi
#define INTERFACE_NAMES ".1.3.6.1.2.1.31.1.1.1.1"

// Get CDP index number with port ifindex number
#define CDP_INDEX ".1.3.6.1.4.1.9.9.23.1.2.1.1.3"

// Get ARP table on the router
#define ARP_LIST ".1.3.6.1.2.1.3.1.1.2"

// /usr/local/include/net-snmp/library/snmp_api.h'de struct variable_list tanimindan alinmistir
struct struct_degerler {
    union { /* value of variable */
        long *integer;
        u_char *string;
        oid *objid;
        u_char *bitstring;
        struct counter64 *counter64;
        #ifdef OPAQUE_SPECIAL_TYPES
		float *floatVal;
        double *doubleVal;
        /* t_union *unionVal; */
        #endif /* OPAQUE_SPECIAL_TYPES */
    }val;
    int last6_oids[6];
};


struct struct_macler {
    char mac[6];
    int vlan;
    int bpn_index[6]; //OID'nin son 6 basamagi
    int bridge_port_numbers;
    int ifindex;
    char interface_names[10];
};

int snmp(char *host, struct ayr *ayarlar);
int IPListAlYaz(struct ayr *ayarlar);
int Ayarlar(struct ayr *ayarlar);
int config_oku(struct ayr *ayarlar);
int snmp_get_and_print(netsnmp_session *ss, oid *theoid, size_t theoid_len, struct struct_degerler *deger);
int snmpwalk(char *myoid, struct struct_degerler *degerler, struct ayr *ayarlar);
void mac(u_char *ptr, char *buf);
void usage(void);
void print_statistics(void);
int database_yaz(struct struct_macler **macler, char *swip, struct ayr *ayarlar, char *zaman);
int calculate_statistics(struct ayr *ayarlar);
char **switch_listesi_al(int *switch_sayisi, struct ayr *ayarlar, char *zaman);
char **router_listesi_al(int *switch_sayisi, struct ayr *ayarlar);
int tracker(struct ayr *ayarlar);
int ip_ekle(char *ip, char *mac, struct ayr *ayarlar);
int arptable(struct ayr *ayarlar);
int ldap_uid_ekle(struct ayr *ayarlar);
int simple_send_mail(char *MTAHost, char *from, char *to, char *subject, char *mesaj);
#endif
